@@ -1,5 +1,5 @@
 ;--------------------------------------------------------------------------
-;  e3.asm v2.6.1 Copyright (C) 2000-05 Albrecht Kleine <kleine@ak.sax.de>
+;  e3.asm v2.7.0 Copyright (C) 2000-06 Albrecht Kleine <kleine@ak.sax.de>
 ;
 ;  see GNU copyright details in e3.asm
 ;--------------------------------------------------------------------------
@@ -14,6 +14,7 @@
 %define LESSWRITEOPS
 ;-------
 %define UTF8				;for UTF8 console or xterm e.g. @ Suse 9.1
+%define UTF8RTS				;runtime detection of UTF8 console display
 ;-------
 %define BEEP_IN_VI			;undef if you hate beeping computers
 %define USE_MATH			;undef if you don't use the numerics
@@ -26,9 +27,23 @@
 ;
 ;	D O   N O T   C H A N G E   B E L O W   L I N E
 ;----------------------------------------------------------------------
+%ifndef LINUX
+ %undef UTF8
+%endif
+
+%ifndef UTF8
+  %undef UTF8RTS
+%endif
+
+%ifdef NETBSD
+%define FREEBSD
+%endif
 %ifdef OPENBSD
 %define FREEBSD
 %endif
+;
+;beware of SYS_... constants > 255! (see NetBSD)
+;
 %ifdef BEOS
 	;posix/termios.h				;termios eq termio
 	%define TERMIOS_SET 8001h			;TCSETA
@@ -252,12 +267,13 @@ db "R/O file system",10			;30
 	%define NCCS 19
 	%define VMIN 6
 ;-------
+%ifndef AMD64
 	;asm/stat.h
 	%define UIDGID_WORD
 	struc stat_struc
 .st_dev:	resd 1
-.st_ino:	resd 1
-.st_mode:	resw 1
+.st_ino:	resd 1		;unsigned long  st_ino;
+.st_mode:	resw 1		;unsigned short st_mode;
 .st_nlink:	resw 1
 .st_uid:	resw 1
 .st_gid:	resw 1
@@ -292,15 +308,67 @@ db "R/O file system",10			;30
 %define SYS_ioctl	54
 %define SYS_dup2	63
 %define SYS_sigaction	67
+%define SYS_rt_sigaction 174
 %define SYS_readlink	85
 %define SYS_fchmod	94
 %define SYS_fchown	95
 %define SYS_fstat	108
 %define SYS_wait4	114
 %define SYS_select	142
-
 %ifndef ARMCPU
 %define time_t	resd
+%endif
+%else				;;--------- AMD64
+	struc stat_struc
+.st_dev:	resq 1
+.st_ino:	resq 1
+.st_nlink:	resq 1
+.st_mode:	resd 1
+.st_uid:	resd 1
+.st_gid:	resd 1
+.__unused0:	resd 1
+.st_rdev:	resq 1
+.st_size:	resq 1
+.st_blksize:	resq 1
+.st_blocks:	resq 1
+.st_atime:	resq 1
+.__unused1:	resq 1
+.st_mtime:	resq 1
+.__unused2:	resq 1
+.st_ctime:	resq 1
+.__unused3:	resq 1
+.__unused4:	resq 1
+.__unused5:	resq 1
+	endstruc
+%define SYS_exit	60
+%define SYS_fork	57
+%define SYS_read	0
+%define SYS_write	1
+%define SYS_open	2
+%define SYS_close	3
+%define SYS_unlink	87
+%define SYS_execve	59
+%define SYS_lseek	8
+%define SYS_utime	132	;30
+%define SYS_kill	62
+%define SYS_rename	82
+%define SYS_pipe	22
+%define SYS_brk		12
+%define SYS_ioctl	16
+%define SYS_dup2	33
+%define SYS_rt_sigaction 13
+%define SYS_rt_sigreturn 15
+%define SYS_readlink	89
+%define SYS_fchmod	91
+%define SYS_fchown	93
+%define SYS_fstat	5
+%define SYS_wait4	61
+%define SYS_select	23	;142
+
+%define time_t	resq
+%endif
+
+%ifndef ARMCPU
 	struc utimbuf_struc
 .actime: time_t  1
 .modtime:time_t 1
@@ -322,8 +390,11 @@ db "R/O file system",10			;30
  ;no more support of CURSORMGNT for UTF-8 thus replaced by:
  ;8.Juni 2004: see /usr/src/linux/Documentation/VGA-softcursor.txt
 %else
- %define CURSORMGNT			;switch cursor depending of 'INSERT'-mode
+ %define CURSORMGNT
  ;trad style
+ %ifdef AMD64
+  %undef CURSORMGNT			;work around January 2006
+ %endif
 %endif
 ;--------
 
@@ -388,11 +459,12 @@ db "Broken pipe",10			;32
 	%endmacro
 %ifdef LIBC
  extern open,read,write,close,lseek,rename,_exit,ioctl,fstat,fchown,select,unlink,fchmod
- extern errno
+ extern _errno
  %undef SYS_readlink
  %undef SYS_brk
  %undef SYS_kill
  %undef SYS_sigaction
+ %undef SYS_rt_sigaction
  %undef USE_PIPE
 %endif
 %ifdef ARMCPU
@@ -420,11 +492,17 @@ db "Broken pipe",10			;32
 	%define ECHO	0x8
 	%define TSize   word			;due oversized ICANON
 ;------
+
 	struc stat_struc
 .st_dev:	resd 1
 .st_ino:	resd 1
-.st_mode:	resw 1
-.st_nlink:	resw 1
+%ifdef OPENBSD
+.st_mode:	resw 1				;for syscall 279
+.st_nlink:	resw 1				;ditto
+%else
+.st_mode:	resd 1
+.st_nlink:	resd 1
+%endif
 .st_uid:	resd 1
 .st_gid:	resd 1
 .st_rdev:	resd 1
@@ -441,6 +519,7 @@ db "Broken pipe",10			;32
 .st_gen:	resd 1
 .st_spare:	resd 5
 	endstruc
+
 %define SYS_exit	1
 %define SYS_fork	2
 %define SYS_read	3
@@ -449,24 +528,34 @@ db "Broken pipe",10			;32
 %define SYS_close	6
 %define SYS_unlink	10
 %define SYS_execve	59
-%define SYS_lseek	19	;also 199
+%define SYS_lseek	199	;old was 19
 %define SYS_utime	138
-%ifndef OPENBSD
 %define SYS_kill	37
-%endif
 %define SYS_rename	128
 %define SYS_pipe	42
 %define SYS_ioctl	54
 %define SYS_dup2	90
-%ifndef OPENBSD
-%define SYS_sigaction	46	;also 342
-%endif
+%define SYS_sigaction	46	;also 342 *** take care if >255 cause only register al used
 %define SYS_readlink	58
 %define SYS_fchown	123
 %define SYS_fchmod      124
-%define SYS_fstat	189
+%define SYS_fstat	279	;old was 189
 %define SYS_wait4	7
 %define SYS_select	93
+
+%ifdef OPENBSD
+%undef SYS_kill
+%undef SYS_sigaction
+%undef  SYS_fstat	
+%define SYS_fstat	189	;could not test 292
+%endif
+
+%ifdef NETBSD
+%undef SYS_kill		;both
+%undef SYS_sigaction	;currently only for old COMPAT layer working
+%endif
+
+
 %define time_t	resd
 
 	struc utimbuf_struc
@@ -687,6 +776,7 @@ c_ospeed speed_t3 1
  %undef CRIPLED_ELF
  %undef UTF8
 %endif
+;--------
 ;
 stdtxtlen	equ 10			;do not move this to EOF: code size would increase
 
@@ -784,9 +874,6 @@ phdr2:					;another Elf32_Phdr starts here
 
 ;-------
 %ifdef AMD64
-;  %undef SYS_sigaction
-;  %undef SYS_kill
-  
   %macro PUSH_ALL 0
   	push rax
 	push rbx
